#ifndef STRING_BUFFER_H_
#define STRING_BUFFER_H_

/*
 * iSCSI driver for Linux
 * Copyright (C) 2002 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: string-buffer.h,v 1.3.4.1 2004/08/10 23:04:49 coughlan Exp $
 *
 * string-buffer.h
 *
 *    variable-sized string buffers
 * 
 */

typedef struct string_buffer {
    size_t allocated_length;
    size_t data_length;		/* not including the trailing NUL */
    char *buffer;
} string_buffer_t;

extern int init_string_buffer(string_buffer_t * s, size_t initial_allocation);
extern string_buffer_t *alloc_string_buffer(size_t initial_allocation);
extern void free_string_buffer(string_buffer_t * s);

extern void enlarge_data(string_buffer_t * s, int length);
extern void remove_initial(string_buffer_t * s, int length);
extern void truncate_buffer(string_buffer_t * s, size_t length);
extern int append_string(string_buffer_t * s, const char *str);
extern int append_sprintf(string_buffer_t * s, const char *format, ...);
extern int adjoin_string(string_buffer_t * s, const char *str);
extern char *buffer_data(string_buffer_t * s);
extern size_t data_length(string_buffer_t * s);
extern size_t unused_length(string_buffer_t * s);
extern void write_buffer(string_buffer_t * s, int fd);

#endif
